from ..patch_helper import PatchHelper


class StateFilter:
    def __init__(self, state: str = "*") -> None:
        self.state = state
        self.__name__ = state + "_state_filter"

    def __call__(_, __, query) -> bool:
        try:
            return _.state == PatchHelper.get_from_pool(query).state.name
        except Exception:
            raise RuntimeError(
                'check storage'
            )


import inspect
from typing import Any, Union
from .patch_data_pool import PatchDataPool
from pyrogram import Client, StopPropagation
from pyrogram.handlers.handler import Handler


# you can modify it
async def create_key(parsed_update, client: Client) -> str:
    chat_id, user_id = "unknown", "unknown"
    if hasattr(parsed_update, "from_user"):
        if parsed_update.from_user is not None:
            user_id = str(parsed_update.from_user.id)
    if hasattr(parsed_update, "chat"):
        if parsed_update.chat is not None:
            chat_id = str(parsed_update.chat.id)
    elif hasattr(parsed_update, "message"):
        if hasattr(parsed_update.message, "chat"):
            if parsed_update.message.chat is not None:
                chat_id = str(parsed_update.message.chat.id)
    return str(client.me.id) + "-" + user_id + "-" + chat_id


class PatchHelper:
    def __init__(self) -> None:
        self.__data = {}
        self.state = "*"

    async def skip_handler(self) -> None:
        """use this method to skip the handler"""
        raise StopPropagation("please ignore this error, it is raised by the PatchHelper.skip_handler method in one of your middlewares")

    async def _get_data_for_handler(self, arguments) -> dict:
        """PLEASE DON'T USE THIS"""
        kwargs = {}
        if "state" in arguments:
            kwargs["state"] = self.state
        if "patch_helper" in arguments:
            kwargs["patch_helper"] = self

        if len(self.__data) > 0:
            for k, v in self.__data.items():
                if k in arguments:
                    kwargs[k] = v
        return kwargs

    async def _process_middleware(self, parsed_update, middleware, client):
        """PLEASE DON'T USE THIS"""
        return await middleware(parsed_update, client, self)

    async def _include_state(self, parsed_update, storage, client):
        """PLEASE DON'T USE THIS"""
        self.state = await storage.checkup(await create_key(parsed_update, client))

    @classmethod
    def get_from_pool(cls, update):
        helper = PatchDataPool.get_helper_from_pool(update)
        if helper is None:
            return cls()
        return helper

    @staticmethod
    def generate_state_key(client_id: int, user_id: Union[int, str] = "unknown", chat_id: Union[int, str] = "unknown") -> str:
        return str(client_id) + "-" + str(user_id) + "-" + str(chat_id)

    @property
    def data(self) -> dict:
        return self.__data

    def __repr__(self) -> str:
        return f"PatchHelper(state: {self.state} | data: {self.__data})"



from pyrogram_patch.fsm.base_storage import BaseStorage
from pyrogram_patch.fsm.states import State


class MemoryStorage(BaseStorage):
    def __init__(self) -> None:
        self.__storage = {}
        self.__data_storage = {}

    async def checkup(self, key) -> "State":
        if key not in self.__storage.keys():
            return State("*", self, key)
        return State(self.__storage[key], self, key)

    async def set_state(self, state: str, key: str) -> None:
        self.__storage[key] = state

    async def set_data(self, data: dict, key: str) -> None:
        if key in self.__data_storage:
            self.__data_storage[key].update(data)
        else:
            self.__data_storage[key] = data

    async def get_data(self, key: str) -> dict:
        if key in self.__data_storage:
            return self.__data_storage[key]
        return {}

    async def finish_state(self, key: str) -> None:
        if key in self.__storage.keys():
            self.__storage.pop(key)
        if key in self.__data_storage.keys():
            self.__data_storage.pop(key)



import inspect
from contextlib import suppress
from typing import Union

import pyrogram
from pyrogram.dispatcher import Dispatcher, log
from pyrogram.handlers import RawUpdateHandler
from .patch_data_pool import PatchDataPool
from pyrogram_patch.fsm import BaseStorage
from pyrogram_patch.middlewares import PatchHelper


class PatchedDispatcher(Dispatcher):
    def __init__(self, client: pyrogram.Client):
        super().__init__(client)
        self.patch_data_pool = PatchDataPool

    async def handler_worker(self, lock):
        while True:
            packet = await self.updates_queue.get()

            if packet is None:
                break

            try:
                update, users, chats = packet
                parser = self.update_parsers.get(type(update), None)

                parsed_updates, handler_type = (
                    await parser(update, users, chats)
                    if parser is not None
                    else (None, type(None))
                )

                if parsed_updates is None:
                    continue

                patch_helper = PatchHelper()
                PatchDataPool.include_helper_to_pool(parsed_updates, patch_helper)

                if PatchDataPool.pyrogram_patch_fsm_storage:
                    await patch_helper._include_state(
                        parsed_updates, PatchDataPool.pyrogram_patch_fsm_storage, self.client
                    )

                # process outer middlewares
                for middleware in PatchDataPool.pyrogram_patch_outer_middlewares:
                    if middleware == handler_type:
                        await patch_helper._process_middleware(
                            parsed_updates, middleware, self.client
                        )

                async with lock:
                    for group in self.groups.values():
                        for handler in group:
                            args = None

                            if isinstance(handler, handler_type):
                                try:
                                    # filtering event
                                    if await handler.check(self.client, parsed_updates):
                                        # process middlewares
                                        for middleware in PatchDataPool.pyrogram_patch_middlewares:
                                            if middleware == type(handler):
                                                await patch_helper._process_middleware(
                                                    parsed_updates,
                                                    middleware,
                                                    self.client,
                                                )
                                        args = (parsed_updates,)
                                except Exception as e:
                                    log.exception(e)
                                    PatchDataPool.exclude_helper_from_pool(parsed_updates)
                                    continue

                            elif isinstance(handler, RawUpdateHandler):
                                try:
                                    # process middlewares
                                    for middleware in PatchDataPool.pyrogram_patch_middlewares:
                                        if middleware == type(handler):
                                            await patch_helper._process_middleware(
                                                    parsed_updates,
                                                    middleware,
                                                    self.client,
                                                )
                                    args = (update, users, chats)
                                except pyrogram.StopPropagation:
                                    PatchDataPool.exclude_helper_from_pool(parsed_updates)
                                    continue
                            if args is None:
                                continue

                            try:
                                # formation kwargs
                                kwargs = await patch_helper._get_data_for_handler(
                                    handler.callback.__code__.co_varnames
                                )
                                if inspect.iscoroutinefunction(handler.callback):
                                    await handler.callback(self.client, *args, **kwargs)
                                else:
                                    args = list(args)
                                    for v in kwargs.values():
                                        args.append(v)
                                    args = tuple(args)
                                    await self.loop.run_in_executor(
                                        self.client.executor,
                                        handler.callback,
                                        self.client,
                                        *args
                                    )
                            except pyrogram.StopPropagation:
                                raise
                            except pyrogram.ContinuePropagation:
                                continue
                            except Exception as e:
                                log.exception(e)
                            finally:
                                PatchDataPool.exclude_helper_from_pool(parsed_updates)
                            break
                    PatchDataPool.exclude_helper_from_pool(parsed_updates)
            except pyrogram.StopPropagation:
                pass
            except Exception as e:
                log.exception(e)



from pyrogram import Client

from .patched_decorators import PatchedDecorators


class Router(PatchedDecorators):
    def __init__(self) -> None:
        self._app = None
        self._decorators_storage: list = []

    def set_client(self, client: Client) -> None:
        self._app = client
        for decorator in self._decorators_storage:
            self._app.add_handler(
                    *decorator
            )


# Copyright (c) 2025 Johnnie
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

from .keyboard_base import Button, InlineButton
from .inline_keyboard import InlineKeyboard
from .reply_keyboard import ReplyKeyboard, ReplyButton, ReplyKeyboardRemove, ForceReply

__version__ = "0.1.2"
__all__ = [
    "Button",
    "InlineButton",
    "InlineKeyboard",
    "ReplyKeyboard",
    "ReplyButton",
    "ReplyKeyboardRemove",
    "ForceReply",
]

__author__ = "Johnnie"


from dataclasses import dataclass
from typing import Union
from pyrogram_patch.fsm import BaseStorage
from contextlib import suppress


@dataclass()
class PatchDataPool:
    update_pool: dict
    pyrogram_patch_middlewares: list
    pyrogram_patch_outer_middlewares: list
    pyrogram_patch_fsm_storage: Union[BaseStorage, None]

    @staticmethod
    def include_helper_to_pool(update, patch_helper) -> None:
        PatchDataPool.update_pool[id(update)] = patch_helper

    @staticmethod
    def exclude_helper_from_pool(update) -> None:
        with suppress(KeyError):
            PatchDataPool.update_pool.pop(id(update))

    @staticmethod
    def get_helper_from_pool(update) -> Union[object, None]:
        return PatchDataPool.update_pool[id(update)]


PatchDataPool.update_pool = {}
PatchDataPool.pyrogram_patch_middlewares = []
PatchDataPool.pyrogram_patch_outer_middlewares = []
PatchDataPool.pyrogram_patch_fsm_storage = None




from abc import ABC, abstractmethod

from .states import State


class BaseStorage(ABC):
    @abstractmethod
    async def checkup(self, key) -> State:
        ...

    @abstractmethod
    async def set_state(self, state: str, key: str) -> None:
        ...

    @abstractmethod
    async def set_data(self, data: dict, key: str) -> None:
        ...

    @abstractmethod
    async def get_data(self, key: str) -> dict:
        ...

    @abstractmethod
    async def finish_state(self, key: str) -> None:
        ...



from . import middlewares
from .middlewares import *

__all__ = [
    "OnMessageMiddleware",
    "OnPoolMiddleware",
    "OnDeletedMessagesMiddleware",
    "OnDisconnectMiddleware",
    "OnRawUpdateMiddleware",
    "OnUserStatusMiddleware",
    "OnInlineQueryMiddleware",
    "OnCallbackQueryMiddleware",
    "OnChosenInlineResultMiddleware",
    "OnEditedMessageMiddleware",
    "OnChatJoinRequestMiddleware",
    "OnChatMemberUpdatedMiddleware",
    "OnUpdateMiddleware",
    "MixedMiddleware"
]



#  Pyrogram - Telegram MTProto API Client Library for Python
#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
#
#  This file is part of Pyrogram.
#
#  Pyrogram is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published
#  by the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  Pyrogram is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with Pyrogram.  If not, see <http://www.gnu.org/licenses/>.

from typing import Callable

import pyrogram

import pyrogram_patch


class OnChatMemberUpdated:
    def on_chat_member_updated(self=None, filters=None, group: int = 0) -> Callable:
        """Decorator for handling event changes on chat members.

        This does the same thing as :meth:`~pyrogram.Client.add_handler` using the
        :obj:`~pyrogram.handlers.ChatMemberUpdatedHandler`.

        Parameters:
            filters (:obj:`~pyrogram.filters`, *optional*):
                Pass one or more filters to allow only a subset of updates to be passed in your function.

            group (``int``, *optional*):
                The group identifier, defaults to 0.
        """

        def decorator(func: Callable) -> Callable:
            if isinstance(self, pyrogram_patch.router.Router):
                if self._app is not None:
                    self._app.add_handler(
                        pyrogram.handlers.ChatMemberUpdatedHandler(func, filters), group
                    )
                else:
                    self._decorators_storage.append((pyrogram.handlers.ChatMemberUpdatedHandler(func, filters), group))
            else:
                raise RuntimeError(
                    "you should only use this in routers, and only as a decorator"
                )

            return func

        return decorator



# Copyright (c) 2025 Johnnie
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

from pyrogram.emoji import *
from pyrogram.types import InlineKeyboardMarkup, InlineKeyboardButton
from dataclasses import dataclass
from .keyboard_base import KeyboardBase, InlineButton
from functools import lru_cache


@dataclass
class InlineKeyboard(InlineKeyboardMarkup, KeyboardBase):
    _PAGINATION_SYMBOLS = {
        "first": "« {}",
        "prev": "‹ {}",
        "current": "· {} ·",
        "next": "{} ›",
        "last": "{} »",
    }

    @staticmethod
    def _get_locales() -> dict[str, str]:
        return {
            "be_BY": f"{FLAG_BELARUS} Беларуская",  # Belarusian - Belarus
            "de_DE": f"{FLAG_GERMANY} Deutsch",  # German - Germany
            "zh_CN": f"{FLAG_CHINA} 中文",  # Chinese - China
            "en_US": f"{FLAG_UNITED_KINGDOM}  English",  # English - United States
            "fr_FR": f"{FLAG_FRANCE} Français",  # French - France
            "id_ID": f"{FLAG_INDONESIA} Bahasa Indonesia",  # Indonesian - Indonesia
            "it_IT": f"{FLAG_ITALY} Italiano",  # Italian - Italy
            "ko_KR": f"{FLAG_SOUTH_KOREA} 한국어",  # Korean - Korea
            "tr_TR": f"{FLAG_TURKEY} Türkçe",  # Turkish - Turkey
            "ru_RU": f"{FLAG_RUSSIA} Русский",  # Russian - Russia
            "es_ES": f"{FLAG_SPAIN} Español",  # Spanish - Spain
            "uk_UA": f"{FLAG_UKRAINE} Українська",  # Ukrainian - Ukraine
            "uz_UZ": f"{FLAG_UZBEKISTAN} Oʻzbekcha",  # Uzbek - Uzbekistan
        }

    def __post_init__(self):
        super().__init__(inline_keyboard=self.keyboard)
        self.callback_pattern = ""
        self.count_pages = 0
        self.current_page = 0

    @staticmethod
    @lru_cache(maxsize=128)
    def _create_button(text: str, callback_data: str) -> InlineKeyboardButton:
        """Create cached button with optimized parameters."""
        return InlineButton(text=text, callback_data=callback_data)

    def paginate(
        self, count_pages: int, current_page: int, callback_pattern: str
    ) -> None:
        """Optimized pagination with better edge case handling.

        Args:
            count_pages (int): Total number of pages.
            current_page (int): The page number currently being viewed.
            callback_pattern (str): The pattern used for callback data.

        Example:
            >>> keyboard = InlineKeyboard()
            >>> keyboard.paginate(10, 1, 'page_{number}')
            >>> print(keyboard.keyboard)
            [[InlineKeyboardButton(text='· 1 ·', callback_data='page_1'), ...]]

        """
        self.count_pages = max(1, count_pages)
        self.current_page = max(1, min(current_page, self.count_pages))
        self.callback_pattern = callback_pattern

        if self.count_pages <= 5:
            pagination = self._build_small_pagination()
        else:
            pagination = self._build_large_pagination()

        self.keyboard.append(pagination)

    def _build_small_pagination(self) -> list[InlineKeyboardButton]:
        return [
            self._create_button(
                text=(
                    self._PAGINATION_SYMBOLS["current"].format(i)
                    if i == self.current_page
                    else str(i)
                ),
                callback_data=self.callback_pattern.format(number=i),
            )
            for i in range(1, self.count_pages + 1)
        ]

    def _build_large_pagination(self) -> list[InlineKeyboardButton]:
        if self.current_page <= 3:
            return self._build_left_pagination()
        elif self.current_page > self.count_pages - 3:
            return self._build_right_pagination()
        return self._build_middle_pagination()

    def _build_left_pagination(self) -> list[InlineKeyboardButton]:
        buttons = []
        for i in range(1, 6):
            if i == self.current_page:
                text = self._PAGINATION_SYMBOLS["current"].format(i)
            elif i == 4:
                text = self._PAGINATION_SYMBOLS["next"].format(i)
            elif i == 5:
                text = self._PAGINATION_SYMBOLS["last"].format(self.count_pages)
                i = self.count_pages
            else:
                text = str(i)
            buttons.append(
                self._create_button(
                    text=text, callback_data=self.callback_pattern.format(number=i)
                )
            )
        return buttons

    def _build_middle_pagination(self) -> list[InlineKeyboardButton]:
        return [
            self._create_button(
                text=self._PAGINATION_SYMBOLS["first"].format(1),
                callback_data=self.callback_pattern.format(number=1),
            ),
            self._create_button(
                text=self._PAGINATION_SYMBOLS["prev"].format(self.current_page - 1),
                callback_data=self.callback_pattern.format(
                    number=self.current_page - 1
                ),
            ),
            self._create_button(
                text=self._PAGINATION_SYMBOLS["current"].format(self.current_page),
                callback_data=self.callback_pattern.format(number=self.current_page),
            ),
            self._create_button(
                text=self._PAGINATION_SYMBOLS["next"].format(self.current_page + 1),
                callback_data=self.callback_pattern.format(
                    number=self.current_page + 1
                ),
            ),
            self._create_button(
                text=self._PAGINATION_SYMBOLS["last"].format(self.count_pages),
                callback_data=self.callback_pattern.format(number=self.count_pages),
            ),
        ]

    def _build_right_pagination(self) -> list[InlineKeyboardButton]:
        buttons = [
            self._create_button(
                text=self._PAGINATION_SYMBOLS["first"].format(1),
                callback_data=self.callback_pattern.format(number=1),
            ),
            self._create_button(
                text=self._PAGINATION_SYMBOLS["prev"].format(self.count_pages - 3),
                callback_data=self.callback_pattern.format(number=self.count_pages - 3),
            ),
        ]

        for i in range(self.count_pages - 2, self.count_pages + 1):
            text = (
                self._PAGINATION_SYMBOLS["current"].format(i)
                if i == self.current_page
                else str(i)
            )
            buttons.append(
                self._create_button(
                    text=text, callback_data=self.callback_pattern.format(number=i)
                )
            )
        return buttons

    def languages(
        self, callback_pattern: str, locales: str | list[str], row_width: int = 2
    ) -> None:
        """Optimized language selection keyboard."""
        if isinstance(locales, str):
            locales = [locales]

        buttons = [
            self._create_button(
                text=self._get_locales().get(locale, "Invalid locale"),
                callback_data=callback_pattern.format(locale=locale),
            )
            for locale in locales
            if locale in self._get_locales()
        ]

        self.keyboard = [
            buttons[i : i + row_width] for i in range(0, len(buttons), row_width)
        ]



# Copyright (c) 2025 Johnnie
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

from dataclasses import dataclass, field
from pyrogram.types import InlineKeyboardButton, WebAppInfo, CallbackGame, LoginUrl


@dataclass
class KeyboardBase:
    row_width: int = 3
    keyboard: list[list] = field(default_factory=list)

    def add(self, *args: object) -> None:
        self.keyboard = [
            list(args[i : i + self.row_width])
            for i in range(0, len(args), self.row_width)
        ]

    def row(self, *args: object) -> None:
        self.keyboard.append(list(args))


@dataclass
class Button:
    text: str

    def __post_init__(self):
        if not isinstance(self.text, str):
            raise ValueError("Button text must be a string")


@dataclass
class InlineButton(Button, InlineKeyboardButton):
    callback_data: str | bytes | None = None
    url: str | None = None
    web_app: WebAppInfo | None = None
    login_url: LoginUrl | None = None
    user_id: int | None = None
    switch_inline_query: str | None = None
    switch_inline_query_current_chat: str | None = None
    callback_game: CallbackGame | None = None
    requires_password: bool | None = None
    pay: bool | None = None
    copy_text: str | None = None

    def __post_init__(self):
        Button.__post_init__(self)
        InlineKeyboardButton.__init__(
            self,
            text=self.text,
            callback_data=self.callback_data,
            url=self.url,
            web_app=self.web_app,
            login_url=self.login_url,
            user_id=self.user_id,
            switch_inline_query=self.switch_inline_query,
            switch_inline_query_current_chat=self.switch_inline_query_current_chat,
            callback_game=self.callback_game,
            requires_password=self.requires_password,
            pay=self.pay,
            copy_text=self.copy_text,
        )



from dataclasses import dataclass


class State:
    def __init__(self, name: str, storage: "BaseStorage", key: str) -> None:
        self.name = name
        self.__storage = storage
        self.__key = key

    async def set_state(self, state) -> None:
        await self.__storage.set_state(state, self.__key)
        self.name = state

    async def set_data(self, data: dict) -> None:
        await self.__storage.set_data(data, self.__key)

    async def get_data(self) -> dict:
        return await self.__storage.get_data(self.__key)

    async def finish(self) -> None:
        await self.__storage.finish_state(self.__key)

    async def create_state(self, key: str) -> "State":
        return await self.__storage.checkup(key)

    def __repr__(self) -> str:
        return f"State(name - {self.name} | key - {self.__key})"

    @property
    def state(self) -> str:
        return self.name


class StateItem:
    def __get__(self, obj, cls):
        for name, obj in vars(cls).items():
            if obj is self:
                return "StatesGroup_" + cls.__name__ + "_State_" + name


@dataclass(init=False, frozen=True)
class StatesGroup:
    pass



# Copyright (c) 2025 Johnnie
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

from dataclasses import dataclass
from pyrogram.types import (
    ReplyKeyboardMarkup,
    KeyboardButton,
    ReplyKeyboardRemove,
    ForceReply,
    RequestChannelInfo,
    RequestChatInfo,
    RequestUserInfo,
    RequestPollInfo,
    WebAppInfo,
)
from .keyboard_base import KeyboardBase, Button


@dataclass
class ReplyKeyboard(ReplyKeyboardMarkup, KeyboardBase):
    is_persistent: bool | None = None
    resize_keyboard: bool | None = None
    one_time_keyboard: bool | None = None
    selective: bool | None = None
    placeholder: str | None = None

    def __post_init__(self):
        super().__init__(
            keyboard=self.keyboard,
            is_persistent=self.is_persistent,
            resize_keyboard=self.resize_keyboard,
            one_time_keyboard=self.one_time_keyboard,
            selective=self.selective,
            placeholder=self.placeholder,
        )


@dataclass
class ReplyButton(KeyboardButton, Button):
    request_contact: bool | None = None
    request_location: bool | None = None
    request_poll: RequestPollInfo | None = None
    request_peer: RequestUserInfo | RequestChannelInfo | RequestChatInfo | None = None
    web_app: WebAppInfo | None = None

    def __post_init__(self):
        super().__post_init__()
        super(KeyboardButton, self).__init__(
            text=self.text,
            request_contact=self.request_contact,
            request_location=self.request_location,
            request_poll=self.request_poll,
            request_peer=self.request_peer,
            web_app=self.web_app,
        )


@dataclass
class ReplyKeyboardRemove(ReplyKeyboardRemove):
    selective: bool | None = None

    def __post_init__(self):
        super().__init__(selective=self.selective)


@dataclass
class ForceReply(ForceReply):
    selective: bool | None = None
    placeholder: str | None = None

    def __post_init__(self):
        super().__init__(selective=self.selective, placeholder=self.placeholder)



from pyrogram.handlers.callback_query_handler import CallbackQueryHandler
from pyrogram.handlers.chat_join_request_handler import ChatJoinRequestHandler
from pyrogram.handlers.chat_member_updated_handler import ChatMemberUpdatedHandler
from pyrogram.handlers.chosen_inline_result_handler import ChosenInlineResultHandler
from pyrogram.handlers.deleted_messages_handler import DeletedMessagesHandler
from pyrogram.handlers.disconnect_handler import DisconnectHandler
from pyrogram.handlers.edited_message_handler import EditedMessageHandler
from pyrogram.handlers.inline_query_handler import InlineQueryHandler
from pyrogram.handlers.message_handler import MessageHandler
from pyrogram.handlers.poll_handler import PollHandler
from pyrogram.handlers.raw_update_handler import RawUpdateHandler
from pyrogram.handlers.user_status_handler import UserStatusHandler


class OnUpdateMiddleware:
    """middleware for all"""

    def __eq__(self, other) -> bool:
        return True


class OnEditedMessageMiddleware:
    """middleware for on_edited_message"""

    def __eq__(self, other) -> bool:
        return other == EditedMessageHandler


class OnUserStatusMiddleware:
    """middleware for on_user_status"""

    def __eq__(self, other) -> bool:
        return other == UserStatusHandler


class OnRawUpdateMiddleware:
    """middleware for on_raw_update"""

    def __eq__(self, other) -> bool:
        return other == RawUpdateHandler


class OnChosenInlineResultMiddleware:
    """middleware for on_chosen_inline_result"""

    def __eq__(self, other) -> bool:
        return other == ChosenInlineResultHandler


class OnDeletedMessagesMiddleware:
    """middleware for on_deleted_messages"""

    def __eq__(self, other) -> bool:
        return other == DeletedMessagesHandler


class OnChatMemberUpdatedMiddleware:
    """middleware for on_chat_member_updated"""

    def __eq__(self, other) -> bool:
        return other == ChatMemberUpdatedHandler


class OnChatJoinRequestMiddleware:
    """middleware for on_chat_join_request"""

    def __eq__(self, other) -> bool:
        return other == ChatJoinRequestHandler


class OnCallbackQueryMiddleware:
    """middleware for on_callback_query"""

    def __eq__(self, other) -> bool:
        return other == CallbackQueryHandler


class OnInlineQueryMiddleware:
    """middleware for on_inline_query"""

    def __eq__(self, other) -> bool:
        return other == InlineQueryHandler


class OnDisconnectMiddleware:
    """middleware for on_disconnect"""

    def __eq__(self, other) -> bool:
        return other == DisconnectHandler


class OnMessageMiddleware:
    """middleware for on_message"""

    def __eq__(self, other) -> bool:
        return other == MessageHandler


class OnPoolMiddleware:
    """middleware for on_pool"""

    def __eq__(self, other) -> bool:
        return other == PollHandler


class MixedMiddleware:
    """pass the types of handlers from pyrogram.handlers during initialization that the malware will process

    patch_manager.include_middleware(CheckIgnoreMiddleware((MessageHandler, EditedMessageHandler), False))


    class CheckIgnoreMiddleware(MixedMiddleware):
    def __init__(self, handlers: tuple, ignore: bool) -> None:
        self.ignore = ignore  # it can be any value you want
        super().__init__(handlers)
    """

    def __init__(self, handlers: tuple) -> None:
        self._handlers = handlers

    def __eq__(self, other) -> bool:
        return other in self._handlers



from json import dumps, loads
from typing import Any, Dict, Optional
from redis.asyncio.client import Redis
from redis.asyncio.connection import ConnectionPool
from pyrogram_patch.fsm.base_storage import BaseStorage
from pyrogram_patch.fsm.states import State


class RedisStorage(BaseStorage):
    def __init__(self, __storage: Redis) -> None:
        self.__storage = __storage

    @classmethod
    def from_url(
            cls, url: str, connection_kwargs: Optional[Dict[str, Any]] = None
    ) -> "RedisStorage":
        if connection_kwargs is None:
            connection_kwargs = {}
        pool = ConnectionPool.from_url(url, **connection_kwargs)
        redis = Redis(connection_pool=pool)
        return cls(__storage=redis)

    async def checkup(self, key) -> "State":
        if await self.__storage.exists(key):
            return State(await self.__storage.get(key), self, key)
        return State("*", self, key)

    async def set_state(self, state: str, key: str) -> None:
        if state is None:
            await self.__storage.delete(key)
        else:
            await self.__storage.set(
                key,
                state
            )

    async def set_data(self, data: dict, key: str) -> None:
        if not data:
            await self.__storage.delete(key)
            return
        await self.__storage.set(
            key,
            dumps(data)
        )

    async def get_data(self, key: str) -> dict:
        if key is None:
            return {}
        if isinstance(key, bytes):
            key = key.decode("utf-8")
        return loads(await self.__storage.get(key))

    async def finish_state(self, key: str) -> None:
        await self.__storage.delete(key)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pykeyboard',
 'pyrogram_patch',
 'pyrogram_patch.fsm',
 'pyrogram_patch.fsm.storages',
 'pyrogram_patch.middlewares',
 'pyrogram_patch.middlewares.middleware_types',
 'pyrogram_patch.router',
 'pyrogram_patch.router.patched_decorators']

package_data = \
{'': ['*']}

install_requires = \
['kurigram>=2.1.35,<3.0.0']

setup_kwargs = {
    'name': 'kurigram-addons',
    'version': '0.1.4',
    'description': 'This package brings together several addons to kurigram including pykeyboard and FSM',
    'long_description': "<img align=center src=./logo.png length=80 width = 400>\n\n\n> This library is a collection of popular Addons and patches for pyrogram/Kurigram.\n> Currently, Pykeyboard and Pyrogram-patch have been added. You're welcome to add more.\n\n# Installation\n\n```shell\npip install kurigram-addons\n```\n\n# Usage\n\nPlease refer to specific readme.md for each addon.\n",
    'author': 'Johnnie',
    'author_email': '99084912+johnnie-610@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pyrogram import Client

from pyrogram_patch.fsm import BaseStorage
from .patch_data_pool import PatchDataPool

from .dispatcher import PatchedDispatcher
from .router import Router


class PatchManager:
    def __init__(self, client: Client):
        self.client = client
        self.dispatcher = client.dispatcher

    def include_middleware(self, middleware: "PatchMiddleware") -> None:
        PatchDataPool.pyrogram_patch_middlewares.append(middleware)

    def include_outer_middleware(self, middleware: "PatchMiddleware") -> None:
        PatchDataPool.pyrogram_patch_outer_middlewares.append(middleware)

    def set_storage(self, storage: BaseStorage) -> None:
        PatchDataPool.pyrogram_patch_fsm_storage = storage

    def include_router(self, router: Router) -> None:
        router.set_client(self.client)


def patch(app: Client) -> PatchManager:
    """app - instance of your pyrogram client
    returns
    MiddlewarePatchManager instance with methods:
    include_middleware and include_outer_middleware
    """
    app.__delattr__("dispatcher")
    app.dispatcher = PatchedDispatcher(app)
    return PatchManager(app)
